# -*- coding: utf-8 -*-

# Coded by Takashi Ida, August 23, 2021

import numpy as np
from scipy import special as sp
from scipy import optimize as opt

##############################################
# inverse scaled complementary error function
#  erfcxx : erfcx(x) (NumPy array)
##############################################
def erfcxinv(erfcxx):
	sqrt_pi = np.sqrt(np.pi)
	x = 1/(erfcxx * sqrt_pi)
	x = x - (sp.erfcx(x) - erfcxx) / (2*x*sp.erfcx(x) - 2/sqrt_pi)
	x = x - (sp.erfcx(x) - erfcxx) / (2*x*sp.erfcx(x) - 2/sqrt_pi)
	x = x - (sp.erfcx(x) - erfcxx) / (2*x*sp.erfcx(x) - 2/sqrt_pi)
	x = x - (sp.erfcx(x) - erfcxx) / (2*x*sp.erfcx(x) - 2/sqrt_pi)
	return x
###################
# Sheared Gaussian
###################
###############################################
# Scaled complementary error function erfcx(b)
# for the shape parmeter (b) to kurtosis (k)
#  erfcxb: erfcx(b) (NumPy array)
######################################
def fSG_erfcxb2k(erfcxb):
	sqrt_pi = np.sqrt(np.pi)
	b = erfcxinv(erfcxb)
	b2 = b**2
	x2 = 0.5 + b2 - b / (sqrt_pi*erfcxb)
	x4 = 0.25*(3 + b2*(12 + b2*4) - 2*b*(5+b2*2)/(sqrt_pi*erfcxb))
	kurtosis = x4 / x2**2 - 3
	return kurtosis
#############################
# kurtosis to erfc(a)
#  k0: kurtosis (NumPy array)
#############################
def fSG_k2erfcxb(kurtosis):
	erfcxb0 = 1.0 # Left side
	erfcxb2 = 0.0 # Right side
	k0 = 0.0 - kurtosis
	k2 = 3.0 - kurtosis
	xTol = 1.0E-8 # Toleance
	nIter = np.ceil(np.log2((erfcxb0-erfcxb2)/xTol)).astype(int)
	for iIter in range(nIter):
		erfcxb1 = 0.5 * (erfcxb0 + erfcxb2)
		k1 = fSG_erfcxb2k(erfcxb1) - kurtosis
		if (np.sign(k0) == np.sign(k1)):
			erfcxb0 = erfcxb1
			k0 = k1
		else:
			erfcxb2 = erfcxb1
			k2 = k1
	return erfcxb1

################
# Demonstration
################
startPoint = 0.0
endPoint = 3.0
step = 0.3
kList = np.arange(startPoint, endPoint, step)
erfcxbList = np.array([])
bList = np.array([])
for i in range(kList.size):
	k = kList[i]
	erfcxb = fSG_k2erfcxb(k)
	b = erfcxinv(erfcxb)
	erfcxbList = np.append(erfcxbList,erfcxb)
	bList = np.append(bList,b)
print ('k = ',kList)
print('erfcx(b) = ', erfcxbList)
print('b = ',bList)
